###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.12 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile, HTML
from Globals import Persistent
from Globals import default__class_init__
from DateTime import DateTime
import Acquisition
import AccessControl.Role
import OFS.Folder
import base_item
import utils
import string
import DateTime
from Products.BookmarkFolder import bookmark
from ZPublisher.HTTPRequest import html_quote

calendar_event_add_redirect = HTMLFile('calendar_event_add_redirect',
					globals())
manage_add_calendar_event_form = HTMLFile('calendar_event_add', globals())

# Bleh
conflicting_events_dialogue = HTML(open('%s/Products/CalendarFolder/calendar_event_conflicting_events_dialogue.dtml' % SOFTWARE_HOME).read())

# These two expand into seconds
expand_hour = lambda x: x * 60 * 60
expand_minute = lambda x: x * 60

def manage_add_calendar_event(self, title='', note='', alarm=0, start=0, end=0,
				#due=0,
				priority=0, status=0, confirm_conflict_ignorance=None,
				REQUEST=None, RESPONSE=None):
	"Add an instance of the calendar_event class."

	new_id = str(self.get_unique_id())

	if REQUEST:

		alarm = expand_hour(REQUEST['alarm_hour']) + \
				expand_minute(REQUEST['alarm_minute'])

		#due = expand_hour(REQUEST['due_hour']) + \
		#		expand_minute(REQUEST['due_minute'])

		start = expand_hour(REQUEST['start_hour']) + \
				expand_minute(REQUEST['start_minute'])

		end = expand_hour(REQUEST['end_hour']) + \
				expand_minute(REQUEST['end_minute'])

		title = REQUEST['title']
		note = REQUEST['note']
		priority = REQUEST['priority']
		status = REQUEST['status']


	conflicting_events_ = self.check_timespan(
		self.get_year().id,
		self.get_month().id, self.id,
		start, end)

	if conflicting_events_ and not confirm_conflict_ignorance:

		if REQUEST is not None:

			return conflicting_events_dialogue(self, REQUEST,
							conflicting_events=conflicting_events_,
							method='manage_add_calendar_event')

		else:

			raise conflicting_events(conflicting_events_)

	
	self._setObject(new_id, calendar_event(
		new_id, title, note, alarm, start, end,# due,
		priority, status))

	if REQUEST is not None and RESPONSE is not None:

		if REQUEST.has_key('__referrer'):

			RESPONSE.redirect(REQUEST['__referrer'])

		else:

			RESPONSE.redirect(self.relative_url())


class conflicting_events:
	"""Exception raised when there are events conflicting."""

	def __init__(self, *args, **kw):
		"""Initiliaizes the exception instance."""

		self.args = args
		self.kw = kw


class calendar_event(
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	meta_type = 'CalendarEvent'
	icon = 'misc_/Calendar/Folder_icon.gif'

	use_local_navigation = 1

	_properties = (
		{'id':'title', 'type':'string'},
		{'id':'note', 'type':'text'},
		{'id':'alarm', 'type':'int'},
		{'id':'start', 'type':'int'},
		{'id':'end', 'type':'int'},
		#{'id':'due', 'type':'int'},
		{'id':'priority', 'type':'int'},
		{'id':'status', 'type':'int'},
		{'id':'alarm_triggered', 'type':'int'},
		{'id':'alarm_muted', 'type':'int'}
		)

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('View CalendarEvents',
		 ('',)),
		)

	#index_html = HTMLFile('calendar_event_index', globals())
	display = HTMLFile('calendar_event_display', globals())

	index_html = base_edit = HTMLFile('calendar_event_edit', globals())

	calendar_event_edit_redirect = HTMLFile('calendar_event_edit_redirect',
						globals())

	priorities = {
		0:'Low',
		1:'Medium',
		2:'High'
	}

	statuses = {
		0:'Completed',
		1:'Pending',
		2:'Cancelled'
		}

	manage_add_bookmark_form = bookmark.manage_add_bookmark_form
	manage_add_bookmark = bookmark.manage_add_bookmark

	meta_types = (
			{'name':'Bookmark', 'action':'manage_add_bookmark_form'},			
			)


	def get_priority_mapping(self):
		"""Returns a mapping of the priorities."""

		priorities = []

		for key in self.priorities.keys():

			priorities.append({'key':key, 'value':self.priorities[key]})

		return priorities


	def get_status_mapping(self):
		"""Returns a mapping of the priorities."""

		statuses = []

		for key in self.statuses.keys():

			priorities.append({'key':key, 'value':self.priorities[key]})

		return statuses


	def get_bookmarks(self):
		"""Returns the bookmarks contained."""

		return self.objectValues('Bookmkark')


	def __init__(self, id, title='', note='', alarm=0,
			start=0, end=0, #due=0,
			priority=0, status='0'):
		"Initialize the class instance"
		
		self.id = id
		self.title = title
		self.note = note
		self.alarm = alarm
		self.start = start
		self.end = end
		#self.due = due
		self.priority = priority
		self.status = status

		self.alarm_triggered = -1
		self.alarm_muted = 0


	def render_html_title(self):
		"""Returns an HTML formatted title."""

		html_title = html_quote(string.strip(self.title))

		self.check_alarm()

		if self.alarm_triggered > -1 and not self.alarm_muted and \
			self.status not in (0,2):

			return """<span style="color: red;">%s (Alarm)</span>""" % \
					(self.title or 'No title')

		else:

			return html_title or 'No title'


	def render_priority(self):
		"""Renders given priority."""

		return self.priorities[self.priority]


	def render_start(self):
		"""Renders the start time."""

		return utils.format_time(self.start)


	def render_end(self):
		"""Renders the end time."""

		return utils.format_time(self.end)


	#def render_due(self):
	#	"""Renders the time the event is due."""

	#	return utils.format_time(self.due)


	def render_status(self):
		"""Renders the status of the event."""

		self.check_alarm()

		return self.statuses[self.status]


	def check_alarm(self):
		"""Checks if alarm is due, if so, sets the indicating variables."""

		current_datetime = DateTime.DateTime()
		event_datetime = DateTime.DateTime('%s/%s/%s %s' % \
					(self.get_date() + (utils.format_time(self.alarm),)))

		if current_datetime > event_datetime:

			self.alarm_triggered = self.alarm


	def reset_alarm(self):
		"""Resets the alarm."""

		self.alarm_triggered = -1
		self.alarm_muted = 0


	def mute_alarm(self):
		"""Mutes the alarm."""

		self.alarm_muted = 1


	def snooze(self, seconds):
		"""Snoozes the alarm."""

		self.alarm_triggered = -1

		#hour_minutes = DateTime.DateTime().TimeMinutes()
		#hour, minutes = string.split(hour_minutes, ':')
		#hour = int(hour)
		#minutes = int(minutes)

		# Test ?FIXME?

		self.alarm = int(DateTime.DateTime()) #hour*60*60 + minutes*60 + self.get_calendar().snooze

		self.check_alarm()
	

	def manage_reschedule(self, REQUEST=None, RESPONSE=None, confirm_conflict_ignorance=None):
		"""Moves an event between days."""

		year = utils.pad_integer(REQUEST['year'])
		month = utils.pad_integer(REQUEST['month'])
		day = utils.pad_integer(REQUEST['day'])

		try:

			DateTime.DateTime('%s-%s-%s' % (year, month, day))

		except DateTime.SyntaxError:

			raise 'InvalidDate', \
				'The date specified is invalid'

		calendar = self.get_calendar()

		try:

			calendar[year]

		except:

			calendar.manage_add_calendar_year(year)


		conflicting_events_ = self.check_timespan(year, month, day,
							 self.start, self.end)

		if conflicting_events_ and not confirm_conflict_ignorance:
		
			return conflicting_events_dialogue(self, REQUEST,
						conflicting_events=conflicting_events_,
						method='manage_reschedule')

		 # So we don't loose the changes we did.

		get_transaction().commit()

		event = self.getParentNode().manage_cutObjects(self.id)
		calendar[year][month][day].manage_pasteObjects(cb_copy_data=event)

		if REQUEST is not None and RESPONSE is not None:

			if REQUEST.has_key('__referrer'):

				RESPONSE.redirect(REQUEST['__referrer'])

			else:

				RESPONSE.redirect(calendar[year][month][day].absolute_url())


	def base_edit_properties(self, REQUEST, confirm_conflict_ignorance=None):
		"""Wrapper that expands hours and minutes into seconds and
		resets the alarm."""

		alarm = expand_hour(REQUEST['alarm_hour']) + \
			expand_minute(REQUEST['alarm_minute'])

		start = expand_hour(REQUEST['start_hour']) + \
			expand_minute(REQUEST['start_minute'])

		end = expand_hour(REQUEST['end_hour']) + \
			expand_minute(REQUEST['end_minute'])

		#due = expand_hour(REQUEST['due_hour']) + \
		#	expand_minute(REQUEST['due_minute'])

		year, month, day = self.get_date()

		conflicting_events_ = []

		for conflicting_event in self.check_timespan(year, month, day, start, end):

			# Tried searching for the instance using the list.index method,
			# but that didn't work for some reason...

			if event.id != self.id:

				conflicting_events_.append(event)

		if conflicting_events_ and not confirm_conflict_ignorance:
		
			return conflicting_events_dialogue(self, REQUEST, conflicting_events=conflicting_events,
								method='base_edit_properties')

		self.alarm = -1
		self.alarm_muted = 0

		self.alarm = alarm
		self.start = start
		self.end = end
		#self.due = due

		self.priority = REQUEST['priority']
		self.status = REQUEST['status']
		self.title = REQUEST['title']
		self.note = REQUEST['note']

		year = utils.pad_integer(REQUEST['year'])
		month = utils.pad_integer(REQUEST['month'])
		day = utils.pad_integer(REQUEST['day'])

		new_day = self.get_calendar()[year][month][day]

		if new_day != self.getParentNode():

			new_day.paste(self.cut())

			#self_cut = self.getParentNode().manage_cutObjects(self.id)
			#new_day.manage_pasteObjects(cb_copy_data=self_cut)

		self.reset_alarm()
		self.check_alarm()

		if REQUEST.has_key('__referrer'):

			REQUEST.RESPONSE.redirect(REQUEST['__referrer'])

		else:

			REQUEST.RESPONSE.redirect(self.getParentNode().absolute_url())


default__class_init__(calendar_event)
