import rfc822_message_parse
import rfc822_message
import rfc822_address
import rfc822_attachment
import cStringIO
import time
import string
import DateTime

priorities = {
		'low':0,
		'medium':1,
		'high':2
	}


def store_message(container, message, readonly=None):
	"""A method that adds an rfc822_message object
	to the container, and possibly address fields.
	It may also add attachments."""

	message_id = str(container.get_unique_id())

	if message.headers.has_key('subject'):

		if not string.strip(message.headers['subject']):

			message.subject = '[No subject]'

	else:

		message.subject = '[No subject]'

	if message.headers.has_key('x-priority'):

		try:

			priority = priorities[string.lower(message.headers['x-priority'])]

		except KeyError:

			priority = 0

	else:

		priority = 0

	rfc822_message.manage_add_rfc822_message(container, message_id,
						sender=message.sender[-1],
						body=message.body,
						subject=message.subject,
						reply_to=message.reply_to,
						priority=priority)


	message_reference = container[message_id]

	if message.attachments:

		for attachment in message.attachments:

			message_reference.manage_add_rfc822_attachment(
				str(container.get_unique_id()),
				data=attachment.data,
				headers=attachment.headers,
				filename=attachment.filename,
				content_type=attachment.media_type + \
					'/' + attachment.sub_type)


	if message.newsgroups:

		message_reference.newsgroups.extend(message.newsgroups)

	for header_name in message.headers.keys():

		message_reference.set_header(header_name, message.headers[header_name])

	for recipient in message.to:

		message_reference['to'].manage_add_rfc822_address(
			str(container.get_unique_id()),
			recipient[-1],
			comment='comment',
			realname='realname')

	for recipient in message.cc:

		message_reference['cc'].manage_add_rfc822_address(
			str(container.get_unique_id()),
			recipient[-1],
			comment='comment',
			realname='realname')

	if readonly:

		message_reference.readonly = 1

	else:

		message_reference.readonly = 0

	message_reference.date_created = DateTime.DateTime()

	message_reference._raw_body = message._raw_message

	return message_reference


def store_unparsed_message(container, unparsed_rfc822_message, readonly=None):
	"""This method lets us parse a message before storing it."""

	if type(unparsed_rfc822_message) is type(''):

		unparsed_rfc822_message = my_string_io(unparsed_rfc822_message)

	message = rfc822_message_parse.rfc822_message_parse(
		unparsed_rfc822_message)

	store_message(container, message, readonly=readonly)


def format_size(bytes, details=None):
	"""Renders a human-readable display of the size."""

	return_string = ""

	if bytes > 1024*1024:

		return_string = return_string + \
			"%.2f MB" % (bytes / 1024.0 / 1024)

	else:

		return_string = return_string + \
			"%.2f KB" % (bytes / 1024.0)

	if details:

		return_string = return_string + " (%s bytes)" % bytes

	return return_string


class my_string_io:
	"""This class represents a wrapper for a cStringIO.StringIO instance,
	giving the ability to use the readlines method."""

	# Should we use a __getattr__ wrapper instead of per-method wrapping?
	# This will prevent breakage of future methods StringIO may have...

	def __init__(self, data=''):

		self._fp = cStringIO.StringIO(data)


	def close(self):

		return self._fp.close()


	def flush(self):

		return self._fp.flush()


	def isatty(self):

		return self._fp.isatty()


	def fileno(self):

		return self._fp.fileno()


	def read(self, size=None):

		if size:

			return self._fp.read(size)

		return self._fp.read()


	def readline(self):

		return self._fp.readline()


	def readlines(self):

		return_list = []

		tmp = None

		while 1:

			tmp = self._fp.readline()

			if tmp:

				return_list.append(tmp)

			else:

				break

		return return_list


	def seek(self, offset, whence=None):

		if whence:

			return self._fp.seek(offset, whence)

		return self._fp.seek(offset)


	def tell(self):

		return self._fp.tell()


	def truncate(self, size=None):

		if size:
			
			return self._fp.truncate(size)

		return self._fp.truncate()


	def write(self, _string):

		return self._fp.write(_string)


	def writelines(self, lines):

		return self._fp.writelines(lines)
