###########################################################################
#
#   Copyright (c) 2000 Morten W. Petersen <morten@esol.no>
#	Copyright (c) 2000 Thingamy Ltd.
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
#    MA 02111-1307 USA
#
#
###########################################################################

# $Author: morten $
# $Revision: 1.6 $


__doc__ = 'Documentation string.'
__version__ = '0.1.0'

from Globals import HTMLFile
from Globals import Persistent
from Globals import default__class_init__
import Acquisition
import AccessControl.Role
import OFS.Folder
import base_item
import string
import DateTime
from Products.BookmarkFolder import bookmark

manage_add_todo_form = HTMLFile('todo_add', globals())

def manage_add_todo(self, title, body, priority, category=None,
				due_year=None, due_month=None, due_day=None,
				REQUEST=None):
	"Add an instance of the todo class"

	new_id = self.get_unique_id()

	category = category or self.get_default_todo_category().id

	if due_year:

		try:

			due_date = DateTime.DateTime("%s/%s/%s" % (due_year, due_month, due_day))
			use_due_date = 1

		except Exception, value:

			print 'exception with due date:', value
			due_date = None
			use_due_date = None

	else:

		due_date = None
		use_due_date = None

	self._setObject(new_id, todo(new_id, title, body, priority, category, use_due_date, due_date))

	if REQUEST is not None:

		REQUEST.RESPONSE.redirect(self.absolute_url())

	else:

		self[new_id]


class todo(
	OFS.Folder.Folder,
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager,
	base_item.base_item
	):
	"Documentation string"

	use_local_navigation = 1

	meta_type = 'Todo'
	icon = 'misc_/TodoFolder/Folder_icon.gif'

	_properties = (
		{'id':'title', 'type':'string'},
		#{'id':'short_body', 'type':'text'},
		{'id':'body', 'type':'text'},
		{'id':'priority', 'type':'int', 'select_widget':'priorities'},
		{'id':'status', 'type':'int', 'select_widget':'statuses'},
		{'id':'category', 'type':'string', 'select_widget':'get_todo_categories'},
		{'id':'local_file', 'type':'string'},
		{'id':'due_date', 'type':'date'},
		)

	__ac_permissions__ = (
		('View management screens',
		 ('manage_tabs', 'manage_main')),
		('Change Permissions',
		 ('manage_access',)),
		('View Todo',
		 ('',)),
		)

	manage_add_todo_form = manage_add_todo_form
	manage_add_todo = manage_add_todo

	manage_add_bookmark_form = bookmark.manage_add_bookmark_form
	manage_add_bookmark = bookmark.manage_add_bookmark

	meta_types = (
			{'name':'Todo', 'action':'manage_add_todo_form'},
			{'name':'Bookmark', 'action':'manage_add_bookmark_form'},			
			)

	base_edit = HTMLFile('todo_edit', globals())
	index_html = HTMLFile('todo_index', globals())

	statuses = {
		0:'Completed',
		1:'Pending',
		2:'Cancelled',
		}

	priorities = {
		0:'Low',
		1:'Medium',
		2:'High'
		}


	def get_bookmarks(self):
		"""Returns a list of bookmarks contained."""

		return self.objectValues('Bookmark')


	def get_todos(self):
		"""Returns a list of the todos contained."""

		return self.objectValues('Todo')


	def get_todo(self):
		"""Returns self."""

		return self


	def __init__(self, id, title, body, priority, category, use_due_date, due_date):
		"Initialize the class instance"

		self.id = id
		self.title = title
		self.body = body
		self.priority = priority
		self.status = 1
		self.category = category
		self.use_due_date = use_due_date
		self.due_date = due_date


	def render_status(self):
		"""Renders the status."""

		return self.statuses[self.status]


	def render_status_widget(self):
		"""Renders an HTML status widget."""

		options = []

		for status_key in self.statuses.keys():

			if status_key == self.status:

				options.append("""<option value="%s" selected>%s</option>""" % \
					(status_key, self.statuses[status_key]))

			else:

				options.append("""<option value="%s">%s</option>""" % \
					(status_key, self.statuses[status_key]))

		return """<select name="status:int">%s</select>""" % string.join(options)


	def render_priority(self):
		"""Renders the priority."""

		return self.priorities[self.priority]


	def render_date(self):
		"""Renders the date this instance is contained within."""

		day = self.get_day()
		month = day.get_month()
		year = month.get_year()

		return "%s/%s/%s" % (day.id, month.id, year.id)


	def render_todo_indentation(self):
		"""Returns the 'indentation' level this todo has, according to parent todos."""

		indentation_html = ''

		for parent_todo in self.get_parent_todos():

			indentation_html = indentation_html + '--&gt;'

		return indentation_html


	def get_parent_todos(self):
		"""Returns a list of parent todos."""

		parent_todos = []

		ref = self.getParentNode()

		while 1:

			if ref.meta_type == 'Todo':

				parent_todos.append(ref)
				ref = ref.getParentNode()

			else:

				break

		return parent_todos


	def base_edit_properties(self, REQUEST, RESPONSE):
		"""Updates the todo and redirects."""

		self.body = REQUEST['body']
		self.priority = REQUEST['priority']
		self.status = REQUEST['status']
		self.category = REQUEST['category']
		self.local_file = REQUEST['local_file']

		if REQUEST.has_key('__referrer'):

			RESPONSE.redirect(REQUEST['__referrer'])

		else:

			RESPONSE.redirect(self.relative_url() + '/base_edit_properties')


default__class_init__(todo)
